import java.io.*;
import java.net.*;
import java.awt.*;
import java.applet.*;
import java.util.*;


class Move  {
	public int	L,R,C;
	public Color	col;

	// constructor
	Move(int l, int r, int c)  {
		L=l;
		R=r;
		C=c;
		col=Color.black;
	}
}

/* *****************************************************************************
*	Class:			UnderlinedButton
*	Purpose:		Button with underlined shortcutLetter
*******************************************************************************/

class UnderlinedButton extends Button  {
	
	public UnderlinedButton(String text)  {
		super(text);
	}
	
	public void setLabel(String label)  {
		super.setLabel(label);
	}
	
	public void paint(Graphics g) {
		
		super.paint(g);
		if (this.isEnabled())  {
				g.setColor(Color.black);
				if (this.getLabel() == "Instructions")	g.fillRect(6,17,4,2);				
				if (this.getLabel() == "Settings") 		g.fillRect(6,17,9,2);
				if (this.getLabel() == "Reset")  		g.fillRect(6,17,9,2);
				if (this.getLabel() == "History")  		g.fillRect(6,17,9,2);
				if (this.getLabel() == "Exit")  		g.fillRect(13,17,8,2);				
		}
	}
}



/* *****************************************************************************
*	Class:			Euler
*	Purpose:		Application class for Euler program
*******************************************************************************/

public  class  Euler extends Applet  implements  Runnable  {

	Thread		background	= null;
	Dimension	size;
	Computer	compute;
	final int	OriginX		= 50,
				OriginY		= 80,
				Delta		= 40,
				Quart		= Delta/4,
				BultOrgX	= OriginX + 9*Delta,
				BultOrgY	= OriginY + 2*Quart,
				Radius		= 5,
				BultStep	= 6*Quart;


	Font		font;
	Color		buttonColor;

	Button		bnSet		= new UnderlinedButton("Settings"),
				bnReset		= new UnderlinedButton("Reset"),
				bnHistory	= new UnderlinedButton("History"),
				bnInfo		= new UnderlinedButton("Instructions"),
				bnExit		= new UnderlinedButton("Exit");
				
					
	Panel		panelButtons 	= new Panel();

	SetFrame	settings	= null;
	InfoFrame	instructions = null;

	Display		display;

	Color		yourColor,
				curColor,
				startColor;
				
	boolean		computer,
				start,
				posChosen = false,
				bultChosen = false,	
				mouseChosen = false,			
				dlgActive = false;

	int			status = 0,
				curR,curC,curL,
				frameBult,
				moveNo,
				remaining,
				level;

	long		lastWakeUp;
	
	Move		moves[];
	int			pattern[][] ;
				

	/* ************************************************************************
	*	Method:			Euler.Euler()
	*	Purpose:		Initiates Euler application
	***************************************************************************/
	
	public Euler()  {

		int		i,j;

		setLayout( new BorderLayout() );
		display		= new Display();
		add( "North", display);
		
		font		= new Font( "Times", Font.BOLD, 12 );
		buttonColor	= new Color(0,255,255);
		setFont(font);

		panelButtons.setBackground(new Color(192,192,192) );
		panelButtons.add(bnInfo );		
		panelButtons.add(bnSet );
		panelButtons.add(bnReset );
		panelButtons.add(bnHistory );
		panelButtons.add(bnExit );
		
		add("South",panelButtons);

		bnInfo.setForeground(Color.black);
		bnSet.setForeground(Color.black);
		bnReset.setForeground(Color.black);
		bnHistory.setForeground(Color.black);	
		bnExit.setForeground(Color.black);	

		bnInfo.setBackground(buttonColor);
		bnSet.setBackground(buttonColor);
		bnReset.setBackground(buttonColor);
		bnHistory.setBackground(buttonColor);
		bnExit.setBackground(buttonColor);

		curR = 0;
		curC = 1;
		moveNo = 1;
		remaining = 16;
		frameBult = -1;

		computer = true;
		start = true ;		
		yourColor = Color.red;
		startColor = Color.red;
		curColor = Color.red;
		level = 8;

		moves = new Move[18];
		for ( i=0; i<=17; i++ )  {
			moves[i] = new Move(0,0,0);
		}
		pattern = new int[5][5];		
		for ( i=0; i<=4; i++)
			for ( j=0; j<=4; j++)
				pattern[i][j]=0;
		compute = new Computer() ;
		display.write("Your move");
		repaint();

		bnSet.requestFocus();

		lastWakeUp = 0 ;

	}	// end Euler()



	
double max(double  a,double b)  {
	if (a>b)
		return a;
	else
		return b;
}

	
double min(double  a,double b)  {
	if (a<b)
		return a;
	else
		return b;
}


	public void init()  {

		requestFocus();
		repaint();
	}

	/* ************************************************************************
	*	Method:			Euler.reset()
	*	Purpose:		Initiates Euler application
	***************************************************************************/
	
	public void reset()  {

		int			i,j;

		curR = 0;
		curC = 1;
		mouseChosen = false;
		moveNo = 1;
		remaining = 16;
		curColor = startColor;
		for ( i=0; i<=17; i++ )  {
			moves[i] = new Move(0,0,0);
		}
		for ( i=0; i<=4; i++)
			for ( j=0; j<=4; j++)
				pattern[i][j]=0;
		if ( computer && start )
			display.write("Your move");
		if ( !computer && startColor==Color.red )
			display.write("Red your move");
		if ( !computer && startColor==Color.blue )
			display.write("Blue your move");
		repaint();
	}


	/* ************************************************************************
	*	Method:			Euler.run()
	*	Purpose:		used for background threads during Thinking period
	***************************************************************************/
	

	public void run()  {
	}


	/* ************************************************************************
	*	Method:			Euler.paintGame()
	*	Purpose:		draws game
	***************************************************************************/
	
	private void paintGame( Color curColor, Graphics g ) {

		int		x0,y0,i;

		// draw plan of game
		g.setColor( new Color(192,192,192) );
		g.fillRect( 0, 0, 450, 310 );
		g.setColor( new Color(0,0,0) );
		x0 = OriginX;
		y0 = OriginY;
		for( i=0; i<=4; i++ )  {
			g.drawLine( x0, y0+i*Delta, x0+4*Delta, y0+i*Delta);
			g.drawLine( x0+i*Delta, y0, x0+i*Delta, y0+4*Delta);
		}
		// draw plan of bults
		x0 = BultOrgX;
		y0 = BultOrgY;
		g.setColor( curColor );
		// bult 1
		g.fillRect( x0-Delta, y0-Quart, Delta, 2*Quart );
		g.fillArc( x0-2*Quart, y0-2*Quart, Delta, Delta, -90, 180 );
		// bult 2
		y0 += BultStep;
		g.fillRect( x0-2*Delta, y0-Quart, 2*Delta, 2*Quart );
		g.fillArc( x0-2*Quart, y0-2*Quart, Delta, Delta, -90, 180 );
		// bult 3
		y0 += BultStep;
		g.fillRect( x0-3*Delta, y0-Quart, 3*Delta, 2*Quart );
		g.fillArc( x0-2*Quart, y0-2*Quart, Delta, Delta, -90, 180 );
		
		y0 = BultOrgY;
		g.setColor(Color.black);
		if ( frameBult != -1 )  {
			g.drawRect( x0 - curL*Delta - 5, y0 + (curL-1)*BultStep - 2*Quart - 5,
			            5 + curL*Delta + 2*Quart + 5, 5 + 4*Quart + 5);
		}
	}


	/* ************************************************************************
	*	Method:			Euler.paintCircle()
	*	Purpose:		draws marker for insertion of bult
	***************************************************************************/
	
	private void paintCircle( int curR, int curC, Graphics g ) {

		int		x0,y0,i;

		g.setColor( new Color(0,0,0) );
		x0 = OriginX + curC*Delta - 2*Quart;
		y0 = OriginY + curR*Delta - 2*Quart;
		g.fillOval( x0-Radius, y0-Radius, 2*Radius, 2*Radius );
	}


	
	/* ************************************************************************
	*	Method:			Euler.paintBults()
	*	Purpose:		draws bults inserted in game sofar
	***************************************************************************/
	
	private void paintBults( Move[] moves, Graphics g ) {

		int		nr = 1,
				x0 = OriginX,
				y0 = OriginY,
				L,R,C;

		while ( moves[nr].L != 0 )  {
			L = moves[nr].L;
			R = moves[nr].R;
			C = moves[nr].C;
			g.setColor(moves[nr].col);
			if ( R==0 )  {
				g.fillRect( x0+(4*C-3)*Quart, y0, 2*Quart, L*Delta );
				g.fillArc( x0+(C-1)*Delta, y0-2*Quart, Delta, Delta, 0, 180 );
			}
			if ( R==5 )  {
				g.fillRect( x0+(4*C-3)*Quart, y0+(4-L)*Delta, 2*Quart, L*Delta );
				g.fillArc( x0+(C-1)*Delta, y0+4*Delta-2*Quart, Delta, Delta, 0, -180 );
			}
			if ( C==0 )  {
				g.fillRect( x0, y0+(4*R-3)*Quart, L*Delta, 2*Quart );
				g.fillArc( x0-2*Quart, y0+(R-1)*Delta, Delta, Delta, 90, 180 );
			}
			if ( C==5 )  {
				g.fillRect( x0+(4-L)*Delta, y0+(4*R-3)*Quart, L*Delta, 2*Quart );
				g.fillArc( x0+4*Delta-2*Quart, y0+(R-1)*Delta, Delta, Delta, -90, 180 );
			}
			if ( L==1 && R>=1 && R<=4 && C>=1 && C<=4 )
				g.fillOval( x0+(C-1)*Delta, y0+(R-1)*Delta, Delta, Delta );
			nr++;
		}
	}



	/* *************************************************************************
	*	Method:			Euler.paint()
	*	Purpose:		display server response
	***************************************************************************/

	public void paint(Graphics g)  {
		
		g.clearRect(0,0,450,350);
		paintGame( curColor, g );
		paintBults( moves, g );
		if ( !mouseChosen )
			paintCircle( curR, curC, g );
	}	// end paint



	/* *************************************************************************
	*	Method:			Euler.action()
	*	Purpose:		Handels button commands
	***************************************************************************/

	public boolean action( Event evt, Object obj)  {		
		
		if ( dlgActive )
			return false;
		if ( evt.target == bnSet )  {
			dlgActive = true;		
			settings = new SetFrame(this);					
		}		
		if ( evt.target == bnHistory )  {
			URL		url = null;
//			try { url = new URL( "file:/C:/PM/PMprojects/Grod/Classes/History.htm"); }
			try { url = new URL( "http://www.PMprojects.net/Grod/Game/History.htm"); }
			catch ( MalformedURLException e) {;}
			getAppletContext().showDocument( url );
		}
		if ( evt.target == bnInfo ) {
			dlgActive = true;		
			instructions = new InfoFrame(this);
		}
		if ( evt.target == bnReset ) {
			reset();
			if ( computer && !start )
				computerMove();						
		}		
		if ( evt.target == bnExit )  {		
			URL		url = null;
//			try { url = new URL( "file:///C:/PM/PMprojects/Web/Index.htm"); }
			try { url = new URL( "http://www.PMprojects.net/Grod/Index.htm"); }
			catch ( MalformedURLException e) {;}
			getAppletContext().showDocument( url );
			Runtime.getRuntime().exit(status);
		}
		
		return true;
	}


	/* *************************************************************************
	*	Method:			Euler.keyDown()
	*	Purpose:		Handels user input
	***************************************************************************/

	public boolean keyDown( Event evt, int key)  {
		
		if ( dlgActive )
			return false;

		if ( key=='S' || key=='s' )  {			
			dlgActive = true;		
			settings = new SetFrame(this);					
			return true ;
		}

		if ( key=='H' || key=='h' )  {			
			URL		url = null;
//			try { url = new URL( "file:///C:/PM/PMprojects/Grod/Classes/History.htm"); }
			try { url = new URL( "http://www.PMprojects.net/Grod/Game/History.htm"); }
			catch ( MalformedURLException e) {;}
			getAppletContext().showDocument( url );
		}
		if ( key=='X' || key=='x' )  {			
			URL		url = null;
//			try { url = new URL( "file:///C:/PM/PMprojects/Web/Index.htm"); }
			try { url = new URL( "http://www.PMprojects.net/Grod/Index.htm"); }
			catch ( MalformedURLException e) {;}
			getAppletContext().showDocument( url );
			Runtime.getRuntime().exit(status);
		}		
		if ( key=='I' || key=='i' )  {			
			dlgActive = true;		
			instructions = new InfoFrame(this);					
			return true ;
		}
		if ( key=='R' || key=='r')   {
			reset();
			if ( computer && !start )
				computerMove();						
			return true;
		}
		if ( evt.when < lastWakeUp )  
			return false;
		if ( key==Event.UP )  { 
			if ( curC==0 || curC==5 )
				curR = ( (curR==1) ? curR : --curR );
			else
				curR = ( (curR==0) ? curR : --curR );
		}
		if ( key==Event.DOWN )  {
			if ( curC==0 || curC==5 )
				curR = ( (curR==4) ? curR : ++curR );
			else
				curR = ( (curR==5) ? curR : ++curR );
		}
		if ( key==Event.LEFT )  {
			if ( curR==0 || curR==5 )
				curC = ( (curC==1) ? curC : --curC );
			else
				curC = ( (curC==0) ? curC : --curC );
		}
		if ( key==Event.RIGHT )  {
			if ( curR==0 || curR==5 )
				curC = ( (curC==4) ? curC : ++curC );
			else
				curC = ( (curC==5) ? curC : ++curC );
		}
		if ( key==Event.DOWN && evt.modifiers==Event.SHIFT_MASK )  {
			curR=5;		
			if (curC==0 || curC==5)  
				curC=1;		
		}
		if ( key==Event.UP && evt.modifiers==Event.SHIFT_MASK )  {
			curR=0;		
			if (curC==0 || curC==5)  
				curC=1;		
		}
		if ( key==Event.LEFT && evt.modifiers==Event.SHIFT_MASK )  {
			curC=0;		
			if (curR==0 || curR==5)  
				curR=1;		
		}
		if ( key==Event.RIGHT && evt.modifiers==Event.SHIFT_MASK )  {
			curC=5;		
			if (curR==0 || curR==5)  
				curR=1;		
		}
		if ( key=='1' )  {			
			curL = 1;
			makeMove();
		}
		if ( key=='2' )  {			
			curL = 2;
			makeMove();
		}
		if ( key=='3' )  {			
			curL = 3;
			makeMove();
		}
		repaint();
		return true;
	}
	
	
	/* *************************************************************************
	*	Method:			mousePosition
	*	Purpose:		Determine area of mouseclick
	***************************************************************************/

	public Move mousePosition(int x, int y)  {
		int		x0,y0,r,c,l;
		
		Move	m = new Move(-1,-1,-1);
		
		r = -1;
		c = -1;
		l = -1;
		x0 = BultOrgX;
		y0 = BultOrgY;
		if ( x >= x0-Delta && x <= x0+2*Quart &&
			 y >= y0 - 2*Quart && y <= y0 + 2*Quart )  
			 	l=1;
		else if ( x >= x0-2*Delta && x <= x0+2*Quart &&
			 y >= y0 + BultStep - 2*Quart && y <= y0 + BultStep +2*Quart )  
			 	l=2;
		else if ( x >= x0-3*Delta && x <= x0+2*Quart &&
			 y >= y0 + 2*BultStep - 2*Quart && y <= y0 + 2*BultStep +2*Quart )  
			 	l=3;
		x0 = OriginX;
		y0 = OriginY;
		if ( x >= x0 - Delta && x <= x0 + 5*Delta ) {
			if ( y >= y0-Delta && y <= y0)
				r=0;
			else if ( y >= y0 && y <= y0 + Delta )
				r=1;
			else if ( y >= y0 + Delta && y <= y0 + 2*Delta )
				r=2;
			else if ( y >= y0 + 2*Delta && y <= y0 + 3*Delta )
				r=3;
			else if ( y >= y0 + 3*Delta && y <= y0 + 4*Delta )
				r=4;
			else if ( y >= y0 + 4*Delta && y <= y0 + 5*Delta )
				r=5;
		}
		if ( y >= y0 - Delta && y <= y0 + 5*Delta ) {
			if ( x >= x0 - Delta && x <= x0)
				c=0;
			else if ( x >= x0 && x <= x0 + Delta )
				c=1;
			else if ( x >= x0 + Delta && x <= x0 + 2*Delta )
				c=2;
			else if ( x >= x0 + 2*Delta && x <= x0 + 3*Delta )
				c=3;
			else if ( x >= x0 + 3*Delta && x <= x0 + 4*Delta )
				c=4;
			else if ( x >= x0 + 4*Delta && x <= x0 + 5*Delta )
				c=5;
		}
		if (  ( r==0 && ( c==0 || c==5) ) || ( r==5 && ( c==0 || c==5) )  )  {
			r = -1;
			c = -1;
		}	
	m = new Move(l,r,c);
	return m;
	}
			
	/* *************************************************************************
	*	Method:			mouseDown
	*	Purpose:		Handels mouse clicks
	***************************************************************************/

	public boolean mouseDown(Event evt, int x, int y)  {
		
		int r,c,l;
		Move 	m;
		
		if ( evt.when<lastWakeUp )
			return false;
		m = mousePosition( x, y);
		r=m.R;
		c=m.C;
		l=m.L;
		if (m.L != -1)  {
			if (!mouseChosen)
				mouseChosen = true;
			curL = m.L;
			bultChosen = true;
			frameBult = m.L;
		}
		if ( m.R != -1 && m.C!=-1)  {
			if ( bultChosen)  {
				curR = m.R;
				curC = m.C;
				frameBult = -1;
				makeMove();
				bultChosen = false;
			}
		}	
		repaint();
		return true;				
	}
	
	

	/* *************************************************************************
	*	Variable:		Waitlevel
	*	Purpose:		Sets thinking time at different levels
	***************************************************************************/

	int	WaitLevel[][]={
					   {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
					   {0,3,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
					   {0,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1},
					   {0,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1},
					   {0,1,1,2,3,2,1,1,1,1,1,1,1,1,1,1,1},
					   {0,1,1,2,2,3,1,1,1,1,1,1,1,1,1,1,1},
					   {0,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1},
					   {0,1,1,1,1,2,2,2,1,1,1,1,1,1,1,1,1},
					   {0,1,1,1,1,1,2,2,3,2,1,1,1,1,1,1,1},
					   {0,1,1,1,1,1,2,2,3,2,1,1,1,1,1,1,1},
					   {0,1,1,1,1,1,1,2,2,3,2,1,1,1,1,1,1},
					   {0,1,1,1,1,1,1,1,2,3,3,2,1,1,1,1,1},
					   {0,1,1,1,1,1,1,1,2,2,2,3,3,1,1,1,1},
					   {0,1,1,1,1,1,1,1,1,1,1,2,2,3,1,1,1},
					   {0,1,1,1,1,1,1,1,1,1,1,1,1,2,3,1,1},
					   {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3},
					   {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1}
					   };



	/* *************************************************************************
	*	Method:			Euler.wait()
	*	Purpose:		Calculate how long to think in msecs depending on 
	*					level and how far into game you are
	***************************************************************************/

	int wait(int level, int remaining)  {

	Random	random = new Random( (new Date()).getTime() );
	double	M[]={0.0,1.0,4.0,6.0},// average in sec. for different wait levels
			S[]={0.0,0.5,1.0,2.0},// spread in sec. for different wait levels
			m,
			s,
			rnd;	// random number in [-0.5 , 0.5 ]


	m=M[WaitLevel[level][remaining]];		
	s=S[WaitLevel[level][remaining]];	
	rnd = min ( s * ( random.nextGaussian() + m ) , 6.0 );
	if (rnd>0) 
		return (int) ( 1000*rnd);
	else
		return 0;
	}



	/* ************************************************************************
	*	Method:			Euler.testMove()
	*	Purpose:		Test if move is legal.
	***************************************************************************/
	
	private boolean testMove( int[][] p, int L, int R, int C ) {

		if ( L==1 && R>=1 && R<=4 && C>=1 && C<=4 && p[R][C]==0 )
			return true;
		else  {
			if (   (R==0 && C>=1 && C<=4) &&
			     ( (L==1 && p[1][C]==0) ||
			       (L==2 && p[1][C]==0	&& p[2][C]==0)  ||
			       (L==3 && p[1][C]==0	&& p[2][C]==0 && p[3][C]==0) ) )
				return true;
			if (   (R==5 && C>=1 && C<=4) &&
				 ( (L==1 && p[4][C]==0) ||
			       (L==2 && p[4][C]==0	&& p[3][C]==0)  ||
			       (L==3 && p[4][C]==0	&& p[3][C]==0 && p[2][C]==0) ) )
				return true;	
			if (   (C==0 && R>=1 && R<=4) &&
			     ( (L==1 && p[R][1]==0) ||
			       (L==2 && p[R][1]==0	&& p[R][2]==0)  ||
			       (L==3 && p[R][1]==0	&& p[R][2]==0 && p[R][3]==0) ) )
				return true;
			if (   (C==5 && R>=1 && R<=4) &&
			( (L==1 && p[R][4]==0) ||
			(L==2 && p[R][4]==0	&& p[R][3]==0)  ||
			(L==3 && p[R][4]==0	&& p[R][3]==0 && p[R][2]==0) ) )
				return true;
		}
		return false;
	}  // testMove


	/* ************************************************************************
	*	Method:			Euler.update()
	*	Purpose:		Updates pattern and move-list
	***************************************************************************/
		
	private void update( int[][] pattern, Move[] moves,
						 int curL, int curR, int curC)  {
 
		moves[moveNo].L = curL;
		moves[moveNo].R = curR;
		moves[moveNo].C = curC;
		moves[moveNo].col= curColor; // curColor;
		moveNo++;
		remaining -= curL;
		if ( curColor == Color.blue )
			curColor = Color.red;
		else
			curColor = Color.blue;
		
		if ( curL==1 && curR>=1 && curR<=4 && curC>=1 && curC<=4 )  
			pattern[curR][curC] = 1;
		if (curR==0)  {
			pattern[1][curC]=1;
			if ( curL==2 || curL==3 )  {
				pattern[2][curC]=1;
				if ( curL==3 )  {
					pattern[3][curC]=1;
				}
			}
		}
		if (curR==5)  {
			pattern[4][curC]=1;
			if ( curL==2 || curL==3 )  {
				pattern[3][curC]=1;
				if ( curL==3 )  {
					pattern[2][curC]=1;
				}
			}
		}
		if (curC==0)  {
			pattern[curR][1]=1;
			if ( curL==2 || curL==3 )  {
				pattern[curR][2]=1;
				if ( curL==3 )  {
					pattern[curR][3]=1;
				}
			}
		}
		if (curC==5)  {
			pattern[curR][4]=1;
			if ( curL==2 || curL==3 )  {
				pattern[curR][3]=1;
				if ( curL==3 )  {
					pattern[curR][2]=1;
				}
			}
		}
	}  // update


	/* ************************************************************************
	*	Method:			Euler.makeMove()
	*	Purpose:		Implements move made by person.
	***************************************************************************/
	
	private void makeMove( ) {

		if ( testMove( pattern, curL, curR, curC ) )  {
			update( pattern, moves, curL, curR, curC );		
			if ( remaining==0 )  {
				if ( computer )
					display.write("You lost on level " + Integer.toString(level));
				if ( !computer && curColor == Color.red )
					display.write("Red player won");					
				if ( !computer && curColor == Color.blue )
					display.write("Blue player won");
			}
			else  {
				if ( computer )
					computerMove();
				if ( !computer && curColor == Color.red )
					display.write("Red your move");					
				if ( !computer && curColor == Color.blue )
					display.write("Blue your move");			
			}
		}
		else
			display.write("Illegal move try again");
		repaint();
	}



	/* ************************************************************************
	*	Method:			Euler.computerMove()
	*	Purpose:		Implements move made by computer
	***************************************************************************/

	public void computerMove()  {

		Graphics	g;

		display.write("I am thinking ....");							
		g = getGraphics();
		paint( g );
		background = new Thread(this);
		int waitTime = wait(level,remaining) ;
		try {Thread.sleep( waitTime );}
		catch (InterruptedException e) {;}
		background.stop();				
		requestMove( pattern, moves, curColor, level);
		requestFocus();							
		if ( remaining==0 )  
			display.write("You won on level " + Integer.toString(level) );				
		else
			display.write("Your move");
		paint( g );
		g.dispose();
		Date dt = new Date();
		lastWakeUp = dt.getTime();
	}


	/* ************************************************************************
	*	Method:			Euler.requestMove()
	*	Purpose:		Implements move made by computer
	***************************************************************************/

	private void requestMove( int[][] pattern, Move[] moves, Color curColor, int level)  {
		
		Int			L = new Int(),
					R = new Int(),
					C = new Int();

		compute.makeMove(pattern, level, L, R, C);
		update( pattern, moves, L.value, R.value, C.value);
	}



	/* ************************************************************************
	*	Method:			Euler.main()
	*	Purpose:		test of communication
	***************************************************************************/

	public static void main(String[] args )  {

		Frame		frame = new Frame("Game of Euler");
		Euler		eulerApp = new Euler();
		
		
		eulerApp.init();
		frame.add("Center",eulerApp);
		frame.show();
		eulerApp.resize( 450,310 );
	}	// end main

}	// end Euler